import socket
import os
import time

def send_file(file_path, client_ip, port):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind(('', port))
    s.listen(1)
    print(f"[TCP] Waiting for receiver on port {port}...")

    # Add small delay to ensure the socket is ready before accepting
    time.sleep(1)

    conn, addr = s.accept()
    print(f"[TCP] Connected to {addr}")

    filename = os.path.basename(file_path)
    filesize = os.path.getsize(file_path)

    # Send filename and filesize, separated by newline
    conn.send(f"{filename}\n{filesize}\n".encode())

    with open(file_path, 'rb') as f:
        while chunk := f.read(1024):
            conn.send(chunk)

    print(f"[TCP] Sent file '{filename}' ({filesize} bytes)")
    conn.close()
    s.close()




def get_filename(file_path):
    return file_path.split("/")[-1]

def get_dirname(dir_path):
    files = dir_path.split("/")
    if files[-1] != "":
        return files[-1]
    else:
        return files[-2]

def get_file_names_from_dir(dir_path):
    files = dir_path.split("/")
    if files[-1] != "":
        files.pop(-1) 
        return files   

    return files



# src/lib/discovery.py

import socket
from akashshare.utils import UDP_PORT, DISCOVERY_MSG, RESPONSE_MSG

def udp_discovery_server():
    udp = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    udp.bind(("", UDP_PORT))
    print(f"UDP Discovery server listening on port {UDP_PORT}")

    while True:
        data, addr = udp.recvfrom(1024)
        if data.decode() == DISCOVERY_MSG:
            print(f"[UDP] Discovery server request from {addr}")
            udp.sendto(RESPONSE_MSG.encode(), addr)
            return addr[0]

def udp_discovery_client():
    udp = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    udp.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
    udp.settimeout(5)

    print("[UDP] Broadcasting for server...")
    udp.sendto(DISCOVERY_MSG.encode(), ('<broadcast>', UDP_PORT))
    try:
        data, addr = udp.recvfrom(1024)
        if data.decode() == RESPONSE_MSG:
            print(f"[UDP] Found server at {addr[0]}")
            return addr[0]
    except socket.timeout:
        print("UDP broadcast timed out.")
    return None



import socket
import time
import sys
from tqdm import tqdm  # progress bar

def spinner():
    # Simple spinner animation generator
    while True:
        for cursor in '|/-\\':
            yield cursor

def start_tcp_receiver(server_ip, port, max_attempts=5, delay=1):
    attempt = 1
    spin = spinner()

    while attempt <= max_attempts:
        sys.stdout.write(f"\r[TCP] Attempt {attempt}: Connecting to {server_ip}:{port} {next(spin)} ")
        sys.stdout.flush()

        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect((server_ip, port))
            sys.stdout.write("\r[TCP] Connected!                      \n")
            sys.stdout.flush()
            break
        except (ConnectionRefusedError, OSError) as e:
            # Instead of printing error directly, just continue spinner & delay
            time.sleep(delay)
            attempt += 1
    else:
        print(f"\n[TCP] Could not connect to sender after {max_attempts} attempts.")
        return

    # Read filename
    filename_bytes = b""
    while not filename_bytes.endswith(b'\n'):
        filename_bytes += s.recv(1)
    filename = filename_bytes.strip().decode()

    # Read filesize
    filesize_bytes = b""
    while not filesize_bytes.endswith(b'\n'):
        filesize_bytes += s.recv(1)
    filesize = int(filesize_bytes.strip().decode())

    with open(filename, 'wb') as f, tqdm(total=filesize, unit='B', unit_scale=True, desc=filename) as pbar:
        while True:
            data = s.recv(1024)
            if not data:
                break
            f.write(data)
            pbar.update(len(data))

    print(f"[TCP] File received and saved as '{filename}'")
    s.close()



from InquirerPy import inquirer
from akashshare.lib import handle_send_action, handle_receive_action, handle_exit_action

def main():  # <- Add this function
    while True:
        choice = inquirer.select(
            message="Choose an action",
            choices=["Send", "Receive", "Exit"],
        ).execute()
        
        if choice == "Send":
            handle_send_action()
        elif choice == "Receive":
            handle_receive_action()
        else:
            handle_exit_action()


if __name__ == "__main__":
    main()



import time  # ADD THIS
from InquirerPy import inquirer
from akashshare.lib.discovery import udp_discovery_server, udp_discovery_client
from akashshare.lib.sender import send_file
from akashshare.lib.receiver import start_tcp_receiver
from akashshare.utils.helper import get_filename, get_dirname, get_file_names_from_dir
import os


def handle_send_action():
    choice = inquirer.select(
        message="What do you want to send?",
        choices=["File", "Directory (coming soon)", "Back"],
    ).execute()

    if choice.lower() == "file":
        file_path = input("Enter absolute file path to send: ").strip()

        if not os.path.isfile(file_path):
            print("[!] Invalid file path.")
            return

        print(f"[INFO] Waiting for receiver to discover us via UDP...")

        receiver_ip = udp_discovery_server()

        if receiver_ip:
            time.sleep(1)  # ADD THIS: Give receiver time to connect
            send_file(file_path, receiver_ip, port=50001)
        else:
            print("[UDP] No receiver found.")

    elif choice.lower().startswith("directory"):
        print("[TODO] Directory sharing is coming soon.")

    else:
        return


def handle_receive_action():
    print("[INFO] Looking for a sender (broadcasting UDP)...")

    sender_ip = udp_discovery_client()
    if sender_ip:
        start_tcp_receiver(server_ip=sender_ip, port=50001)
    else:
        print("[UDP] No sender found.")


def handle_exit_action():
    print("Happy Sharing! Bye 👋")
    exit(0)
